#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    :platform: Unix
    :synopsis: Constants used in this module.

"""

# Status keys
ERROR_TIME_AXIS_OK = '000'
ERROR_TIME_AXIS_KO = '001'
ERROR_TIME_BOUNDS_INS = '002'
ERROR_TIME_BOUNDS_AVE = '003'
ERROR_TIME_BOUNDS_KO = '004'
ERROR_TIME_UNITS = '005'
ERROR_TIME_CALENDAR = '006'
ERROR_END_DATE_IN_VS_NAME = '007a'
ERROR_END_DATE_NAME_VS_IN = '007b'
ERROR_END_DATE_NAME_VS_REF = '008a'
ERROR_END_DATE_REF_VS_NAME = '008b'
ERROR_END_DATE_IN_VS_REF = '009a'
ERROR_END_DATE_REF_VS_IN = '009b'
ERROR_START_DATE_IN_VS_NAME = '010a'
ERROR_START_DATE_NAME_VS_IN = '010b'

ERROR_CORRECTED_SET = (ERROR_TIME_AXIS_KO,
                       ERROR_TIME_BOUNDS_INS,
                       ERROR_TIME_BOUNDS_KO,
                       ERROR_TIME_UNITS,
                       ERROR_TIME_CALENDAR,
                       ERROR_END_DATE_IN_VS_NAME,
                       ERROR_END_DATE_NAME_VS_IN,
                       ERROR_END_DATE_REF_VS_IN,
                       ERROR_END_DATE_IN_VS_REF,
                       ERROR_END_DATE_REF_VS_NAME,
                       ERROR_END_DATE_NAME_VS_REF)

# Status messages
STATUS = {ERROR_TIME_AXIS_OK: 'Time axis seems OK',
          ERROR_TIME_AXIS_KO: 'Incorrect time axis over one or several time steps',
          ERROR_TIME_BOUNDS_INS: 'An instantaneous time axis should not embed time boundaries',
          ERROR_TIME_BOUNDS_AVE: 'An averaged time axis should embed time boundaries',
          ERROR_TIME_BOUNDS_KO: 'Incorrect time bounds over one or several time steps',
          ERROR_TIME_UNITS: 'Time units must be unchanged for the same dataset',
          ERROR_TIME_CALENDAR: 'Calendar must be unchanged for the same dataset',
          ERROR_END_DATE_IN_VS_NAME: 'End date in file is earlier than end date from filename',
          ERROR_END_DATE_NAME_VS_IN: 'End date in file is late than end date from filename',
          ERROR_END_DATE_NAME_VS_REF: 'End date from filename is earlier than theoretical end date',
          ERROR_END_DATE_REF_VS_NAME: 'End date from filename is later than theoretical end date',
          ERROR_END_DATE_IN_VS_REF: 'End date in file is earlier than theoretical end date',
          ERROR_END_DATE_REF_VS_IN: 'End date in file is later than theoretical end date',
          ERROR_START_DATE_IN_VS_NAME: 'Start date in file is earlier than start date from filename',
          ERROR_START_DATE_NAME_VS_IN: 'Start date in file is later than start date from filename'}

# List of variable required by each process
PROCESS_VARS = ['pattern',
                'ref_units',
                'ref_calendar',
                'ref_start',
                'ref_end',
                'write',
                'force',
                'on_fly',
                'limit',
                'lock',
                'progress',
                'nbfiles',
                'ignore_codes']

# Number of decimal to keep in axis truncation
NDECIMALS = 8



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    :platform: Unix
    :synopsis: Constants used in this package.

"""
from datetime import datetime as dt

# Program version
VERSION = '4.6.4'

# Date
VERSION_DATE = dt(year=2018, month=12, day=14).strftime("%Y-%d-%m")

# Cards name
RUN_CARD = 'run.card'
CONF_CARD = 'config.card'

# Filedef directory format
FILEDEF_ROOT = '/ccc/work/cont003/igcmg/igcmg/IGCM'
FILEDEF_DIRECTORY_FORMAT = '{root}/CMIP6/{longname}/{modelname}/{experimentname}/{member}/{year}'

# Shell colors map
SHELL_COLORS = {'red': 1,
                'green': 2,
                'yellow': 3,
                'blue': 4,
                'magenta': 5,
                'cyan': 6,
                'gray': 7}

# Default time units
DEFAULT_TIME_UNITS = {'cordex': 'days since 1949-12-01 00:00:00',
                      'cordex-adjust': 'days since 1949-12-01 00:00:00'}

# Climatology file suffix
CLIM_SUFFIX = '-clim.nc'

# Frequency increment
FREQ_INC = {('None', 'subhr'): [30, 'minutes'],
            ('None', 'subhrPt'): [30, 'minutes'],
            ('None', '1hr'): [1, 'hours'],
            ('None', '1hrCM'): [1, 'hours'],
            ('None', '1hrPt'): [1, 'hours'],
            ('None', '3hr'): [3, 'hours'],
            ('None', '3hrPt'): [3, 'hours'],
            ('None', '6hr'): [6, 'hours'],
            ('None', '6hrPt'): [6, 'hours'],
            ('None', 'day'): [1, 'days'],
            ('None', 'dec'): [10, 'years'],
            ('None', 'mon'): [1, 'months'],
            ('None', 'monC'): [1, 'months'],
            ('None', 'monPt'): [1, 'months'],
            ('None', 'yr'): [1, 'years'],
            ('None', 'yrPt'): [1, 'years'],
            ('3hr', '3hr'): [3, 'hours'],
            ('3hr', '3hrPt'): [3, 'hours'],
            ('3hrCurt', '3hr'): [3, 'hours'],
            ('3hrMlev', '3hr'): [3, 'hours'],
            ('3hrPlev', '3hr'): [3, 'hours'],
            ('3hrSlev', '3hr'): [3, 'hours'],
            ('6hrLev', '6hrPt'): [6, 'hours'],
            ('6hrLev', '6hr'): [6, 'hours'],
            ('6hrPlev', '6hr'): [6, 'hours'],
            ('6hrPlevPt', '6hr'): [6, 'hours'],
            ('6hrPlevPt', '6hrPt'): [6, 'hours'],
            ('Aclim', 'monClim'): [1, 'months'],
            ('AERday', 'day'): [1, 'days'],
            ('AERhr', '1hr'): [1, 'hours'],
            ('AERmon', 'mon'): [1, 'months'],
            ('AERmonZ', 'mon'): [1, 'months'],
            ('aero', 'mon'): [1, 'months'],
            ('Amon', 'mon'): [1, 'months'],
            ('Amon', 'monC'): [1, 'months'],
            ('Amon', 'monClim'): [1, 'months'],
            ('AmonExtras', 'mon'): [1, 'months'],
            ('CF3hr', '3hrPt'): [3, 'hours'],
            ('cf3hr', '3hr'): [3, 'hours'],
            ('CFday', 'day'): [1, 'days'],
            ('cfDay', 'day'): [1, 'days'],
            ('CFmon', 'mon'): [1, 'months'],
            ('cfMon', 'mon'): [1, 'months'],
            ('cfOff', 'mon'): [1, 'months'],
            ('cfSites', '3hr'): [3, 'hours'],
            ('cfSites', '6hr'): [6, 'hours'],
            ('cfSites', 'subhr'): [30, 'minutes'],
            ('CFsubhr', 'subhrPt'): [30, 'minutes'],
            ('day', 'day'): [1, 'days'],
            ('dayExtras', 'day'): [1, 'days'],
            ('E1hr', '1hr'): [1, 'hours'],
            ('E1hr', '1hrPt'): [1, 'hours'],
            ('E1hrClimMon', '1hrCM'): [1, 'hours'],
            ('E3hr', '3hr'): [3, 'hours'],
            ('E3hrPt', '3hrPt'): [3, 'hours'],
            ('E6hrZ', '6hr'): [6, 'hours'],
            ('E6hrZ', '6hrPt'): [6, 'hours'],
            ('Eday', 'day'): [1, 'days'],
            ('EdayZ', 'day'): [1, 'days'],
            ('Emon', 'mon'): [1, 'months'],
            ('EmonZ', 'mon'): [1, 'months'],
            ('Esubhr', 'subhrPt'): [30, 'minutes'],
            ('Eyr', 'yr'): [1, 'years'],
            ('Eyr', 'yrPt'): [1, 'years'],
            ('ImonAnt', 'mon'): [1, 'months'],
            ('ImonGre', 'mon'): [1, 'months'],
            ('IyrAnt', 'yr'): [1, 'years'],
            ('IyrGre', 'yr'): [1, 'years'],
            ('Lclim', 'monClim'): [1, 'months'],
            ('LIclim', 'monClim'): [1, 'months'],
            ('LImon', 'mon'): [1, 'months'],
            ('Lmon', 'mon'): [1, 'months'],
            ('Oclim', 'monClim'): [1, 'months'],
            ('Oclim', 'monC'): [1, 'months'],
            ('Oday', 'day'): [1, 'days'],
            ('Odec', 'dec'): [10, 'years'],
            ('OIclim', 'monClim'): [1, 'months'],
            ('OImon', 'mon'): [1, 'months'],
            ('Omon', 'mon'): [1, 'months'],
            ('OmonExtras', 'mon'): [1, 'months'],
            ('Oyr', 'yr'): [1, 'years'],
            ('OyrExtras', 'yr'): [1, 'years'],
            ('SIday', 'day'): [1, 'days'],
            ('SImon', 'mon'): [1, 'months'],
            ('SImon', 'monPt'): [1, 'months'],
            ('sites', 'subhr'): [30, 'minutes'],
            ('HOMAL3hrPt', '3hrPt'): [3, 'hours'],
            ('HOMAL3hrPt', '3hr'): [3, 'hours'],
            ('HOMALmon', 'mon'): [1, 'months'],
            ('HOMEPmon', 'mon'): [1, 'months'],
            ('HOMOImon', 'mon'): [1, 'months']}

# Known time units
TIME_UNITS = {'s': 'seconds',
              'm': 'minutes',
              'h': 'hours',
              'D': 'days',
              'M': 'months',
              'Y': 'years'}

# Frequencies to consider in case of non-instant time correction
AVERAGE_CORRECTION_FREQ = ['day', 'mon', 'monPt', 'yr', 'yrPt', '1hrCM']

# Climatology frequencies
CLIMATOLOGY_FREQ = ['monC', 'monClim', '1hrCM']

# Available CF calendars
CALENDARS = ['gregorian',
             'standard',
             'proleptic_gregorian',
             'noleap',
             'julian',
             '365_day',
             'all_leap',
             '366_day',
             '360_day']

TIME_UNITS_FORMAT = "{seconds,minutes,hours,days} since YYYY-MM-DD [HH:mm:ss]"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    :platform: Unix
    :synopsis: Constants used for help message.

"""

from custom_print import *

# Help
TITLE = COLOR('yellow').bold("""_________________________
.___ ___| |_ _ _____ ___.
| | | __| __| | . . | -_|
|_|_|___|_| |_|_|_|_|___|""")

INTRO = """netCDF files describe all required dimensions to work with. Dimensions such as longitude, latitude and time are included in netCDF files as vectors. Time is a key dimension that could lead to flawed studies or unused data if misdeclared. "nctime" allows to easily diagnose the time definition of netCDF files to ensure a proper analysis."""

URL = COLOR('gray').italic("""See full documentation and references at http://prodiguer.github.io/nctime/""")

DEFAULT = COLOR('white').italic("""The default values are displayed next to the corresponding flags.""")

PROGRAM_DESC = {
    'axis': """
{}

{} The time axis is a key dimension. Unfortunately, this time axis often is mistaken in files from coupled climate models and leads to flawed studies or unused data. Consequently, these files cannot be used or, even worse, produced erroneous results, due to problems in the time axis description. The netCDF Time Axis Checker ("nctxck") allows to easily check the time axis correctness by rebuilding a convention-compliant time axis for NetCDF files.

{}

{}""".format(TITLE, INTRO, URL, DEFAULT),
    'overlap': """
{}

{}  The scheme of chunked files in archive designs is not fixed and depends on several parameters (the institute, the model, the frequency, etc.). These different schemes lead to unnecessary overlapping files with a more complex folder reading, wasting disk space or broken time series. The netCDF Time Coverage Checker (nctcck) allows to easily analyse and correct the time series continuity (i.e., broken time series, overlapping files). 

{}

{}""".format(TITLE, INTRO, URL, DEFAULT)}

EPILOG = COLOR('gray').italic("""Developed by:
    Levavasseur, G. (UPMC/IPSL - glipsl@ipsl.fr)
    Laliberte, F.   (ExArch - frederic.laliberte@utoronto.ca)""")

OPTIONAL = COLOR('blue')("""Optional arguments""")

POSITIONAL = COLOR('magenta')("""Positional arguments""")

HELP = """Show this help message and exit.

"""

VERSION_HELP = """Program version.

"""

PROJECT_HELP = """Project name corresponding to a section of the configuration file.
If not submitted project id is deduced from first netCDF file scanned.

"""

INI_HELP = """Initialization/configuration directory containing "esg.<project>.ini" files.
If not specified, the environment variable ESGINI_DIR is used.
If not exist usual datanode directory is used.

"""

LOG_HELP = """Logfile directory.
If submitted without value, the default directory is: $PWD/logs.
If not, standard output is used.

"""

VERBOSE_HELP = """Verbose mode.

"""

ALL_HELP = """Display all results.
Default only shows error(s).

"""

INPUT_HELP = """One or more variable directories or files to diagnose.
Unix wildcards are allowed.

"""

DIRECTORY_HELP = """One or more variable directories to diagnose.
Unix wildcards are allowed.

"""

RESOLVE_HELP = """Resolves overlapping files.
THIS ACTION DEFINITELY MODIFY INPUT FILES!

"""

FULL_ONLY_HELP = """Removes only full overlapping files.
THIS ACTION DEFINITELY MODIFY INPUT FILES!

"""

ON_FLY_HELP = """Ignore the test on end date consistency for on going simulation (this also include completed files).

"""

XML_HELP = """One or several directory with DR2XML files used for the simulation.

"""

CARD_HELP = """The libIGCM directory with "run.card" and "config.card" of the simulation.
This option is IPSL-specific.

"""

START_HELP = {
    'overlap': """Starting timestamp for the covered period.
Default is the minimum of all timestamps among filenames.

""",
    'axis': """Theoretical starting timestamp to consider.
Default is to consider timestamps from filename.

"""}

END_HELP = {
    'overlap': """Theoretical ending timestamp to consider.
Default is to consider timestamps from filename.

""",
    'axis':
        """Theoretical ending timestamp to consider.
Default is to consider timestamps from filename.

"""}

LIMIT_HELP = """Limit of displayed wrong timesteps (if exist).
Default is set to 5. If submitted without value, all timesteps are printed.

"""

IGNORE_ERROR_HELP = """One or several error codes to ignore comma-separated.

"""

WRITE_HELP = """Rewrites time axis depending on checking.
THIS ACTION DEFINITELY MODIFY INPUT FILES!

"""

FORCE_HELP = """Forces time axis rewriting.
THIS ACTION DEFINITELY MODIFY INPUT FILES!

"""

MAX_PROCESSES_HELP = """Number of maximal processes to simultaneously treat several files. Max is the CPU count.
Set to 1 seems sequential processing.
Set to -1 uses the max CPU count.
Default is set to 4 processes.

"""

IGNORE_DIR_HELP = """Filter directories NON-matching the regular expression.
Default ignore paths with folder name(s) starting with "." pattern.
(Regular expression must match from start of path; prefix with ".*" if required.)

"""

INCLUDE_FILE_HELP = """Filter files matching the regular expression.
Duplicate the flag to set several filters.
Default only include NetCDF files.

    """

EXCLUDE_FILE_HELP = """Filter files NON-matching the regular expression.
Duplicate the flag to set several filters.
Default only exclude hidden files (with names not
starting with "."). It excludes fixed data without
time axis in any case.

"""

SET_INC_HELP = """Overwrites the default time increment of a table:frequency couple.
Duplicate the flag to overwrite several increments.
"all" keyword is allowed to overwrite increment for all frequencies or all tables (e.g., all:mon=2M will set monthly frequency equivalent to 2 Months between time steps instead of 1 for all tables).
Available units are: s, m, h, D, M, Y respectively for seconds, minutes, hours, days, months, years.

"""

CALENDAR_HELP = """Defines reference calendar for the check.
Default is to consider the calendar from the first file scanned.
Available calendars are those from CF conventions:
gregorian, standard, proleptic_gregorian, noleap, 365_day, all_leap, 366_day, 360_day.

"""

UNITS_HELP = """Defines reference time units for the check.
Default is to consider the time units from the first file scanned.
Available time units format is "<units> since YYYY-MM-DD HH:mm:ss" where <units> stands for seconds, minutes, hours or days.

"""

COLOR_HELP = """Enable colors. (Default is to enable when writing to a terminal.)

"""

NO_COLOR_HELP = """Disable colors.

"""
